//  Copyright 2019 Kakao Corp.
//
//  Licensed under the Apache License, Version 2.0 (the "License");
//  you may not use this file except in compliance with the License.
//  You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
//  Unless required by applicable law or agreed to in writing, software
//  distributed under the License is distributed on an "AS IS" BASIS,
//  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//  See the License for the specific language governing permissions and
//  limitations under the License.

import UIKit
import KakaoSDKCommon
import KakaoSDKAuth

@UIApplicationMain
class AppDelegate: UIResponder, UIApplicationDelegate {

    var window: UIWindow?


    func application(_ application: UIApplication, didFinishLaunchingWithOptions launchOptions: [UIApplication.LaunchOptionsKey: Any]?) -> Bool {
        
        
        KakaoSDKCommon.shared.initSDK(appKey: "e19a463823bdbefb87c2c66c3fb6ab59", loggingEnable:true)

        // Override point for customization after application launch.
        window = UIWindow(frame: UIScreen.main.bounds)

        let rootViewController = SDKSampleViewController()
        
        rootViewController.setTitle()
        rootViewController.navigationItem.rightBarButtonItems = [UIBarButtonItem(title: "Log", style: UIBarButtonItem.Style.plain, target: rootViewController, action: #selector(rootViewController.showLog))]
        let navigationController = UINavigationController(rootViewController:rootViewController)
        rootViewController.view.backgroundColor = UIColor.red
        window?.rootViewController = navigationController
        window?.makeKeyAndVisible()
        
        return true
    }

    func applicationWillResignActive(_ application: UIApplication) {
        // Sent when the application is about to move from active to inactive state. This can occur for certain types of temporary interruptions (such as an incoming phone call or SMS message) or when the user quits the application and it begins the transition to the background state.
        // Use this method to pause ongoing tasks, disable timers, and invalidate graphics rendering callbacks. Games should use this method to pause the game.
    }

    func applicationDidEnterBackground(_ application: UIApplication) {
        // Use this method to release shared resources, save user data, invalidate timers, and store enough application state information to restore your application to its current state in case it is terminated later.
        // If your application supports background execution, this method is called instead of applicationWillTerminate: when the user quits.
    }

    func applicationWillEnterForeground(_ application: UIApplication) {
        // Called as part of the transition from the background to the active state; here you can undo many of the changes made on entering the background.
    }

    func applicationDidBecomeActive(_ application: UIApplication) {
        // Restart any tasks that were paused (or not yet started) while the application was inactive. If the application was previously in the background, optionally refresh the user interface.
    }

    func applicationWillTerminate(_ application: UIApplication) {
        // Called when the application is about to terminate. Save data if appropriate. See also applicationDidEnterBackground:.
    }
    
    func application(_ app: UIApplication, open url: URL, options: [UIApplication.OpenURLOptionsKey : Any] = [:]) -> Bool {
        // TODO: v1에 있던 isKakaolinkCallback, isStoryPostCallback, isLoginCallback 제공
        //       v1과 동일하게 할 경우 customRedirectUri 사용할 때 isLoginCallback이 좀 어색한거 같음
        //       위 기능들을 제공하되 인터페이스 형태는 좀 다양하게 생각해봐야 할 듯.. kakaoAppScheme + kakaolinkHost|storyPostHost|authorizeRedirectHost 형태로 한다던가..
        //       API 공통으로 URL 핸들링을 해야해서 메소드를 AuthController 말고 SDKCommon에 구현하는 것도 좋을 듯
        // TODO: 카카오링크->앱링크, 스토리포스팅->앱링크 execParam 값 확인할 수 있는 샘플코드
        return AuthController.handleOpenUrl(url: url, options: options)
    }
}

extension AppDelegate {
    static func templateId() -> (message:Int64, link:Int64) {
            return (message:19388, link:19389)
    }
}

extension SDKSampleViewController {

    override open var canBecomeFirstResponder: Bool {
        return true
    }

    override open func motionBegan(_ motion: UIEvent.EventSubtype, with event: UIEvent?) {
        super.motionBegan(motion, with: event)

        if (motion == .motionShake) {
            self.showLog()
        }
    }

    @objc public func showLog() {
        let window = UIApplication.shared.keyWindow!

        let debugVC = SDKDebugViewController.shared
        let debugNaviController = SDKDebugNavigationController(rootViewController:debugVC)
        debugNaviController.modalPresentationStyle = .overCurrentContext
        debugNaviController.modalTransitionStyle = .crossDissolve

        if let modalViewController = window.rootViewController?.presentedViewController {
            if (modalViewController is SDKDebugNavigationController) {
                modalViewController.dismiss(animated: false, completion: nil)
            }
            else {
                modalViewController.present(debugNaviController, animated: false) {
                    debugVC.showLog()
                }
            }
        }
        else {
            window.rootViewController!.present(debugNaviController, animated: false) {
                debugVC.showLog()
            }
        }
    }
}
